package de.ozgcloud.apilib.vorgang.grpc;

import java.util.List;

import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextAttachingInterceptor;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgang;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangId;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangQuery;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangService;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangStub;
import de.ozgcloud.apilib.vorgang.Page;
import de.ozgcloud.vorgang.vorgang.GrpcFilterBy;
import de.ozgcloud.vorgang.vorgang.GrpcFindVorgangRequest;
import de.ozgcloud.vorgang.vorgang.GrpcFindVorgangRequest.GrpcOrderBy;
import de.ozgcloud.vorgang.vorgang.GrpcFindVorgangWithEingangRequest;
import de.ozgcloud.vorgang.vorgang.VorgangServiceGrpc.VorgangServiceBlockingStub;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.client.inject.GrpcClient;

@RequiredArgsConstructor
public class GrpcOzgCloudVorgangService implements OzgCloudVorgangService {

	@GrpcClient("vorgang-manager")
	private final VorgangServiceBlockingStub vorgangServiceStub;

	private final OzgCloudVorgangMapper mapper;
	private final OzgCloudVorgangStubMapper stubMapper;
	private final OzgCloudVorgangQueryMapper queryMapper;

	private final OzgCloudCallContextProvider contextProvider;

	@Override
	public OzgCloudVorgang getById(@NonNull OzgCloudVorgangId vorgangId) {
		var response = getVorgangServiceStub().findVorgangWithEingang(buildRequest(vorgangId));

		return mapper.fromGrpc(response.getVorgangWithEingang());
	}

	private GrpcFindVorgangWithEingangRequest buildRequest(@NonNull OzgCloudVorgangId vorgangId) {
		return GrpcFindVorgangWithEingangRequest.newBuilder()
				.setId(vorgangId.toString())
				.build();
	}

	@Override
	public List<OzgCloudVorgangStub> search(@NonNull String searchString) {
		return search(searchString, Page.startPage());
	}

	@Override
	public List<OzgCloudVorgangStub> search(@NonNull String searchString, Page page) {
		var response = getVorgangServiceStub().findVorgang(buildSearchRequest(searchString, page));

		return response.getVorgangList().stream().map(stubMapper::fromGrpc).toList();
	}

	GrpcFindVorgangRequest buildSearchRequest(@NonNull String searchString, Page page) {
		return GrpcFindVorgangRequest.newBuilder()
				.setFilterBy(GrpcFilterBy.newBuilder().build())
				.setSearchBy(searchString)
				.setOffset(page.getOffset())
				.setLimit(page.getLimit())
				.setOrderBy(GrpcOrderBy.CREATED_AT_DESC)
				.build();
	}

	@Override
	public List<OzgCloudVorgangStub> find(OzgCloudVorgangQuery query, Page page) {
		var response = getVorgangServiceStub().findVorgang(buildFindRequest(query, page));

		return response.getVorgangList().stream().map(stubMapper::fromGrpc).toList();
	}

	GrpcFindVorgangRequest buildFindRequest(OzgCloudVorgangQuery query, Page page) {
		return GrpcFindVorgangRequest.newBuilder()
				.setLimit(page.getLimit())
				.setOffset(page.getOffset())
				.setQuery(queryMapper.toGrpc(query))
				.setOrderBy(GrpcOrderBy.CREATED_AT_DESC)
				.build();
	}

	VorgangServiceBlockingStub getVorgangServiceStub() {
		return vorgangServiceStub.withInterceptors(new OzgCloudCallContextAttachingInterceptor(contextProvider));
	}
}
